#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-ce-config v. 2.4.8
#
# Post-installation script for configuring the gLite Computing Element Service
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
# Mario Reale  < mario.reale@cern.ch >
# Leanne Guy  < leanne.guy@cern.ch >
#
# Version info: $Id$
#
# Usage: python glite-ce-config [-c|-v|-h|--help]
#        -c                                    print configuration
#        -v                                    print version
#        -h,--help                             print usage info
#        --configure                           configure the service
#        --ldif                                update the LCG Providers ldif file and exit
#        --start                               start the service
#        --stop                                stop the service
#        --status                              print service status (0 = running, 1 = idle)
#        --service                             specify a service to which the start, stop and status commands are applied
#                                              Possible values are:
#                                              - all (default)
#                                              - cemon (CE Monitor)
#                                              - ll (LB Locallogger)
#                                              - gk (Gatekeeper)
#                                              - dgas (DGAS Client, if enabled)
#                                              - st (R-GMA Service Publisher)
#                                              - condor (only applies to stop command)
#        --siteconfig=<site config URL>        use remote site configuration file from URL
#        --verbose                             print verbose output (overrides config file)
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,pwd,grp,time,re
import sys
import getopt
import commands
import _xmlplus
import _xmlplus.xpath as xpath
import xml.dom.minidom
from xml.dom.minidom import Node
from xml.dom.ext import PrettyPrint
import urllib
import socket

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
from  gridSecurity import gridSecurity

from condor import Condor
from globus import Globus
from tomcat5 import Secure_tomcat5 as Tomcat5
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                        # all config values from the XML file
global condor                        # the one and only condor service instance
global globus                        # the one and only globus service instance
global tomcat                        # the one and only globus service instance
global rgmaServicetool

class glite_ce:

    def __init__(self):
        self.verbose = 0
        self.iptables_config = 1
        self.version = "2.4.8"
        self.name = "glite-ce"
        self.friendly_name = "gLite Computing Element"
        self.servicecommand = 'all'
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -n, --noiptables    disables the automatic iptables configuration'
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the service'
        print '    --ldif              update the LCG Providers ldif file and exit'
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            print service status (0 = running, 1 = idle)'
        print '    --service           specify a service to which the start, stop and status commands are applied'
        print '                        Possible values are:'
        print '                        - all (default)'
        print '                        - cemon (CE Monitor)'
        print '                        - ll (LB Locallogger)'
        print '                        - gk (Gatekeeper)'
        print '                        - dgas (DGAS Client, if enabled)'
        print '                        - st (R-GMA Service Publisher)'
        print '                        - condor (only applies to stop command)'
        print '    --siteconfig=<URL>  use site configuration file at URL'
        print '    --verbose           print verbose output (overrides config file)'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        
        # Set default error level
        error_level = 0
        
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------
        
        if glib.checkJava() != 0:
            return 1
        
        #----------------------------------------
        # Parameter checks
        #----------------------------------------
        
        if not params.has_key('cemon.lrms'):
            params['cemon.lrms'] = "pbs"
        else:
            if params['cemon.lrms'] == "":
                params['cemon.lrms'] = "pbs"
                
        if not params['cemon.lrms'] in ['pbs','torque','lsf','condor']:
            glib.printErrorMessage("Unrecognised LRMS type. Cannot continue")
            return 1
            
        if params['cemon.cetype'] == 'condorc':
           params['cemon.cetype'] = 'condor'
           
        if params['cemon.lrms'] == 'torque':
           params['cemon.lrms'] = 'pbs'
           
        if params['cemon.lrms'] == 'condor' and params['cemon.cetype'] != 'condor':
            glib.printErrorMessage("The Condor batch system is supported only with a Condor CE type")
            glib.printErrorMessage("Please check the configuration file")
            return 1

        if not params.has_key('log.parser.address'):
            params['log.parser.address'] = "127.0.0.1"
            params['log.parser.hostname'] = glib.fq_hostname
        else:
            if params['log.parser.address'] == "":
                params['log.parser.address'] = "127.0.0.1"
                params['log.parser.hostname'] = glib.fq_hostname
            else:
                try:
                    params['log.parser.address'] = socket.gethostbyname(params['log.parser.address'])
                    params['log.parser.hostname'] = socket.getfqdn(socket.gethostbyaddr(params['log.parser.address'])[0])
                except Exception, ex:
                    glib.printErrorMessage("The log.parser.address param with value %s" %params['log.parser.address'])
                    glib.printErrorMessage("cannot be resolved to a valid hostname or IP address")
                    glib.printErrorMessage("Configuration cannot continue")

        #-------------------------------------------------------------------------------
        # Configure Tomcat
        #-------------------------------------------------------------------------------
        
        print "\nConfiguring Tomcat ...                                  "
        
        # stop tomcat first
        tomcat.stop()
        
        # configure tomcat
        errorcode = tomcat.configure()
        if errorcode != 0:
            print "Configuring Tomcat                                             ",
            glib.printFailedMessage()
            return 1
             
        print "Configuring Tomcat                                           ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis CE will be configured to serve the following VOs:"
        for vo in glib.getVoList():
            print "=> %s" % vo
            
        #-------------------------------------------------------------------------------
        # Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
        #-------------------------------------------------------------------------------
        
        ret_val = gsec.configure_mkgridmap()
        if ret_val == 1:
            return 1
        elif ret_val == 2:
            error_level = 2

        #-------------------------------------------------------------------------------
        # Configure pool accounts
        #-------------------------------------------------------------------------------
        
        ret_val = gsec.create_pool_accounts()
        if ret_val == 1:
            return 1

        #-------------------------------------------------------------------------------
        # Create the sgm account
        #-------------------------------------------------------------------------------
        
        if gsec.addSgmAccounts():
            return 1
            
        #-------------------------------------------------------------------------------
        # Update the gridmap file with VOMS FQAN entries:
        #-------------------------------------------------------------------------------
    
        if gsec.addFQAN2Gridmapfile():
            return 1

        # Add WMS host entries
        if params.has_key('cemon.wms.host.subject'):

            if self.verbose:
                print "Adding WMS host entries..."
    
            for idx in xrange(len(params['cemon.wms.host.subject'])):
    
                if params['cemon.wms.host.subject'][idx].strip() != "":
                    file = open(params['host.gridmapfile'], 'r' )
                    for line in file.xreadlines():
                        if line.find('"%s"' % params['cemon.wms.host.subject'][idx]) != -1:
                            file.close()
                            break
                    else:
                        file.close()
                        file = open(params['host.gridmapfile'], 'a')
                        file.write('\n"%s" %s\n' % (params['cemon.wms.host.subject'][idx],params['lb.user']))
                        file.close()

        if self.verbose:
            print "Updating the grid-mapfile %s                " % params['host.gridmapfile'],
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the group map file /etc/grid-security/groupmapfile :
        #-------------------------------------------------------------------------------
    
        if gsec.createGroupmapfile():
            return 1

        #-------------------------------------------------------------------------------
        # Create vomsdir and check for VOMS certificates
        #-------------------------------------------------------------------------------

        if gsec.createVomsdir():
            return 1

        #-------------------------------------------------------------------------------
        # Create the $GLITE_LOCATION/etc/vomses file 
        #-------------------------------------------------------------------------------
        
        if gsec.createVomses():
            return 1
            
        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1

        print "\nInstalling gLite Security Utilities                        ",
        glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Create  the $GLITE_LOCATION/grid-services directory
        #-------------------------------------------------------------------------------
        
        if self.verbose:
            print "Creating the %s/etc/grid-services directory..." % os.environ['GLITE_LOCATION']
    
        path = '%s/etc/grid-services' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            os.mkdir(path,0755)
            os.chown(path,0,0)
    
        if self.verbose:
            print "Creating the %s/etc/grid-services directory     " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the gatekeeper config file $GLITE_LOCATION/etc/gatekeeper.conf
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Creating the %s/etc/gatekeeper.conf file..." % os.environ['GLITE_LOCATION']
        
        path = "%s/etc/gatekeeper.conf" % os.environ['GLITE_LOCATION']
        path1 = "%s/etc/gatekeeper.conf.1" % os.environ['GLITE_LOCATION']
        if os.path.exists(path1):
            os.remove(path1)
        if os.path.exists(path):
            os.rename(path,path1)
        file = open(path, 'w')
        file.write("-x509_cert_dir %s\n" % params['ca.certificates.dir'])
        file.write("-x509_user_cert %s\n" % params['host.certificate.file'])
        file.write("-x509_user_key %s\n" % params['host.key.file'])
        file.write("-gridmap %s\n" % params['host.gridmapfile'])
        file.write("-home %s\n" % os.environ['GLITE_LOCATION'])
        file.write("-e libexec\n")
        file.write("-port %s\n" % params['gatekeeper.port'])
        file.write("-grid_services %s/etc/grid-services\n" % os.environ['GLITE_LOCATION'])
        file.write('-logfile "%s/gatekeeper.log"\n' % os.environ['GLITE_LOCATION_LOG'])
        file.write("-lcas_db_file lcas.db\n")
        file.write("-lcas_etc_dir %s/etc/lcas/\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcasmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcmaps_db_file lcmaps.db\n")
        file.write("-lcmaps_etc_dir %s/etc/lcmaps\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcmapsmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
        file.close()
        
        if self.verbose:
            print "Creating the %s/etc/gatekeeper.conf file         " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Create the gridftp config file $GLITE_LOCATION/etc/gridftpd_lcas_lcmaps.conf
        #-------------------------------------------------------------------------------
    
        if params['ce.gridftp.enable']=='true':
            if self.verbose:
                print "Creating the %s/etc/gridftpd_lcas_lcmaps.conf file..." % os.environ['GLITE_LOCATION']
            
            path = "%s/etc/gridftpd_lcas_lcmaps.conf" % os.environ['GLITE_LOCATION']
            path1 = "%s/etc/gridftpd_lcas_lcmaps.conf.1" % os.environ['GLITE_LOCATION']
            if os.path.exists(path1):
                os.remove(path1)
            if os.path.exists(path):
                os.rename(path,path1)
            file = open(path, 'w')
            file.write("-lcas_db_file lcas.db\n")
            file.write("-lcas_etc_dir %s/etc/lcas/\n" % os.environ['GLITE_LOCATION'])
            file.write("-lcasmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
            file.write("-lcmaps_db_file lcmaps.db.gridftp\n")
            file.write("-lcmaps_etc_dir %s/etc/lcmaps\n" % os.environ['GLITE_LOCATION'])
            file.write("-lcmapsmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
            file.write("-lcas_debug_level 0\n")
            file.write("-lcmaps_debug_level 0\n")
            file.write("-lcmaps_policies_acq voms_acq,standard_acq\n")
            file.write("-lcmaps_policies_enf voms_enf,standard_enf")
            file.close()
            
            if self.verbose:
                print "Creating the %s/etc/gridftpd_lcas_lcmaps.conf file         " % os.environ['GLITE_LOCATION'],
                glib.printOkMessage()
    
            #-------------------------------------------------------------------------------
            # Create the LCMAPS DB file $GLITE_LOCATION/etc/lcmaps/lcmaps.db.gridftp : 
            #-------------------------------------------------------------------------------
        
            lcmaps_db_file = '%s/etc/lcmaps/lcmaps.db.gridftp' % os.environ['GLITE_LOCATION']
            lcmaps_db_file1 = '%s/etc/lcmaps/lcmaps.db.gridftp.1' % os.environ['GLITE_LOCATION']
        
            if self.verbose:
                print "Creating the gridftp LCMAPS DB file %s..." % lcmaps_db_file
        
            if os.path.exists(lcmaps_db_file1):
                os.remove(lcmaps_db_file1)
            if os.path.exists(lcmaps_db_file):
                os.rename(lcmaps_db_file,lcmaps_db_file1)
    
            if not os.path.exists(os.path.dirname(lcmaps_db_file)):
                os.makedirs(os.path.dirname(lcmaps_db_file))
    
            file = open(lcmaps_db_file, 'w')
            file.write('path = %s/lib/modules\n' % os.environ['GLITE_LOCATION'] ) 
            file.write('posix_enf = "lcmaps_posix_enf.mod"\n')
            file.write('" -maxuid 1"\n')
            file.write('" -maxpgid 1"\n')
            file.write('" -maxsgid 32"\n')
            file.write(' \n')
            file.write('localaccount = "lcmaps_localaccount.mod"\n')
            file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
            file.write(' \n')
            file.write('poolaccount = "lcmaps_poolaccount.mod"\n')
            file.write('" -override_inconsistency"\n')
            file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
            file.write('" -gridmapdir %s"\n'% params['host.gridmap.dir'])
            file.write(' \n')
            file.write('vomsextract = "lcmaps_voms.mod"\n')
            file.write('" -vomsdir %s"\n' % os.environ['X509_VOMS_DIR'])
            file.write('" -certdir %s"\n' % params['ca.certificates.dir'])
            file.write('\n')
            file.write('vomslocalgroup = "lcmaps_voms_localgroup.mod"\n')
            file.write('" -groupmapfile %s"\n' % params['host.groupmapfile'])
            file.write('" -mapmin 1"\n')
            file.write('\n')
            file.write('vomslocalaccount = "lcmaps_voms_localaccount.mod"\n')
            file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'])
            file.write('" -use_voms_gid"\n')
            file.write('\n')
            file.write('vomspoolaccount = "lcmaps_voms_poolaccount.mod"\n')
            file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'])
            file.write('" -gridmapdir %s"\n' % params['host.gridmap.dir'])
            file.write('" -override_inconsistency"\n')
            file.write('good             = "lcmaps_plugin_example.mod"\n')
            file.write('# policies\n')
            file.write('voms_acq:\n')
            file.write('vomsextract -> vomslocalgroup\n')
            file.write('vomslocalgroup -> vomspoolaccount\n')
            file.write('vomspoolaccount -> good | vomslocalaccount\n')
    
            file.write('standard_acq:\n')
            file.write('poolaccount -> good | localaccount\n')
            file.write('localaccount -> good\n')
    
            file.write('voms_enf:\n')
            file.write('vomsextract -> vomslocalgroup\n')
            file.write('vomslocalgroup -> vomspoolaccount\n')
            file.write('vomspoolaccount -> posix_enf | vomslocalaccount\n')
            file.write('vomslocalaccount -> posix_enf\n')
    
            file.write('standard_enf:\n')
            file.write('poolaccount -> posix_enf | localaccount\n')
            file.write('localaccount -> posix_enf\n')
            
            file.close()
            
            if not os.path.exists(lcmaps_db_file):
                    glib.printFailedMessage()
                    glib.printErrorMessage("Failed to create %s" % lcmaps_db_file)
                    return 1
        
            if self.verbose:
                    print "Creating the LCMAPS DB file %s              " % lcmaps_db_file,
                    glib.printOkMessage()
         

                
        #-------------------------------------------------------------------------------
        # Create the jobmanager-fork file $GLITE_LOCATION/etc/grid-services/jobmanager-fork:
        #-------------------------------------------------------------------------------

        if self.verbose:
            print "Creating the %s/etc/grid-services/jobmanager-fork file and link..." % os.environ['GLITE_LOCATION']
    
        jobmanager_fork = "%s/etc/grid-services/jobmanager-fork" % os.environ['GLITE_LOCATION']
        jobmanager_fork1 = "%s/etc/grid-services/jobmanager-fork.1" % os.environ['GLITE_LOCATION']
        if os.path.exists(jobmanager_fork1):
            os.remove(jobmanager_fork1)
        if os.path.exists(jobmanager_fork):
            os.rename(jobmanager_fork,jobmanager_fork1)
        file = open(jobmanager_fork, 'w')
        file.write("stderr_log,local_cred - %s/libexec/globus-job-manager \
globus-job-manager -conf %s/etc/globus-job-manager.conf -type fork -rdn jobmanager-fork \
-machine-type unknown -publish-jobs \n" % ( os.environ['GLOBUS_LOCATION'], os.environ['GLITE_LOCATION'] ))
        file.close()
    
        if not os.path.exists(jobmanager_fork):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s \n" % jobmanager_fork)
            return 1
    
        jobmanager_link = '%s/etc/grid-services/jobmanager' % os.environ['GLITE_LOCATION']
    
        if os.path.exists(jobmanager_link):
            os.remove(jobmanager_link)
    
        os.symlink(jobmanager_fork, jobmanager_link)
    
        if not os.path.exists(jobmanager_link):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create symlink s \n" % jobmanager_link)
            return 1
    
        if self.verbose:
            print "Creating the %s/etc/grid-services/jobmanager-fork file and link    " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()
           
        #-------------------------------------------------------------------------------
        # Create the  globus job manager config file $GLITE_LOCATION/etc/globus-job-manager.conf :  
        #-------------------------------------------------------------------------------
    
        globus_jobmanager_conf = '%s/etc/globus-job-manager.conf' % os.environ['GLITE_LOCATION']
        globus_jobmanager_conf1 = '%s/etc/globus-job-manager.conf.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the globus job manager config file %s..." % globus_jobmanager_conf
    
        if os.path.exists(globus_jobmanager_conf1):
            os.remove(globus_jobmanager_conf1)
        if os.path.exists(globus_jobmanager_conf):
            os.rename(globus_jobmanager_conf,globus_jobmanager_conf1)
    
        osname = os.uname()[2]
        if params.has_key('globus.osversion'):
            if params['globus.osversion'] != "":
                osname = params['globus.osversion']
        
        # Host DN subject
        if params.has_key('globus.hostdn'):
            if params['globus.hostdn'] != "":
                gdn = params['globus.hostdn']
            else:
                try:
                    if os.path.exists(params['host.certificate.file']):
                        gdn = glib.get_certificate_subject(params['host.certificate.file'])
                    else:
                        glib.printFailedMessage()
                        glib.printErrorMessage("Could not find a valid host certificate at %s" % params['host.certificate.file'])
                        return 1
                except AttributeError, e:
                    print e
                    return 1
        else:
            try:
                if os.path.exists(params['host.certificate.file']):
                    gdn = glib.get_certificate_subject(params['host.certificate.file'])
                else:
                    glib.printFailedMessage()
                    glib.printErrorMessage("Could not find a valid host certificate at %s" % params['host.certificate.file'])
                    return 1
            except AttributeError, e:
                print e
                return 1

        file = open(globus_jobmanager_conf, 'w')
        file.write('-home "%s" \n' % os.environ['GLOBUS_LOCATION'])
        file.write("-globus-gatekeeper-host %s\n" %  glib.fq_hostname)
        file.write("-globus-gatekeeper-port %s\n" % params['gatekeeper.port'])
        file.write('-globus-gatekeeper-subject "%s"\n' % gdn)
        file.write("-globus-host-cputype %s\n" % os.uname()[4])
        file.write("-globus-host-manufacturer pc\n")
        file.write("-globus-host-osname %s\n"  % os.uname()[0])
        file.write('-globus-host-osversion "%s"\n' % osname)
        file.write("-save-logfile on_error\n")
        file.write("-state-file-dir %s/gram_job_state\n" % os.environ['GLITE_LOCATION_VAR'])
        file.write("-machine-type unknown\n")
        if os.environ.has_key("GLOBUS_TCP_PORT_RANGE"):
            file.write('-globus-tcp-port-range "%s"\n' % os.environ['GLOBUS_TCP_PORT_RANGE'])
        file.close()
        
        if not os.path.exists(globus_jobmanager_conf):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % globus_jobmanager_conf)
            return 1
    
        if self.verbose:
            print "Creating the globus job manager config file %s     " % globus_jobmanager_conf,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create  the $GLITE_LOCATION_TMP dir and set permissions:
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Creating directory %s and setting permissions..."  % os.environ['GLITE_LOCATION_TMP']
    
        os.system('/bin/mkdir -p %s' % os.environ['GLITE_LOCATION_TMP'])
        if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % os.environ['GLITE_LOCATION_TMP'])
            return 1
        os.system('/bin/chmod 01777 %s' % os.environ['GLITE_LOCATION_TMP'])
    
        if self.verbose:
            print "Creating directory %s and setting permissions    "  % os.environ['GLITE_LOCATION_TMP']
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create  the  $GLITE_LOCATION_VAR/gram_job_state dir and set corresponding permissions:
        #-------------------------------------------------------------------------------
    
        gram_job_state_dir = '%s/gram_job_state' % os.environ['GLITE_LOCATION_VAR']
    
        if self.verbose:
            print "Creating directory %s and setting permissions..." % gram_job_state_dir
    
        os.system('/bin/mkdir -p %s ' % gram_job_state_dir)
        if not os.path.exists(gram_job_state_dir):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % gram_job_state_dir)
            return 1
        os.system('/bin/chmod 01777 %s' %  gram_job_state_dir)
            
        if self.verbose:
            print "Creating directory %s and setting permissions     " % gram_job_state_dir,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the $GLITE_LOCATION/etc/lcas/lcas.db file :
        #-------------------------------------------------------------------------------
    
        lcas_db_file = '%s/etc/lcas/lcas.db' % os.environ['GLITE_LOCATION']
        lcas_db_file1 = '%s/etc/lcas/lcas.db.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the %s file..." % lcas_db_file,
     
        if not os.path.exists(os.path.dirname(lcas_db_file)):
            os.makedirs(os.path.dirname(lcas_db_file))
    
        if os.path.exists(lcas_db_file1):
            os.remove(lcas_db_file1)
        if os.path.exists(lcas_db_file):
            os.rename(lcas_db_file,lcas_db_file1)
    
        file = open(lcas_db_file, 'w')
        
        file.write('pluginname=lcas_userban.mod,pluginargs=ban_users.db\n')
        file.write('pluginname=lcas_voms.mod,pluginargs="-vomsdir %s \
-certdir %s \
-authfile %s \
-authformat simple \
-use_user_dn" \n' \
            % (os.environ['X509_VOMS_DIR'], params['ca.certificates.dir'], params['host.gridmapfile']) )
            
        file.close()
        
        if not os.path.exists(lcas_db_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % lcas_db_file)
            return 1
    
        if self.verbose:
            print "Creating the %s file                         " % lcas_db_file,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create an empty banned users  $GLITE_LOCATION/etc/lcas/ban_users.db file
        #-------------------------------------------------------------------------------
    
        banned_users_file = '%s/etc/lcas/ban_users.db' % os.environ['GLITE_LOCATION']
    
        if not os.path.exists(banned_users_file):
            if self.verbose:
                print "Creating an empty banned users file in %s to start with..." % banned_users_file
            os.system('/bin/touch %s' % banned_users_file),
        else:
            if self.verbose:
                print "Banned user file %s already exists  " %  banned_users_file
    
        if not os.path.exists(banned_users_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % banned_users_file)
            return 1
    
        #-------------------------------------------------------------------------------
        # Create the LCMAPS DB file $GLITE_LOCATION/etc/lcmaps/lcmaps.db : 
        #-------------------------------------------------------------------------------
    
        lcmaps_db_file = '%s/etc/lcmaps/lcmaps.db' % os.environ['GLITE_LOCATION']
        lcmaps_db_file1 = '%s/etc/lcmaps/lcmaps.db.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the LCMAPS DB file %s..." % lcmaps_db_file
    
        if os.path.exists(lcmaps_db_file1):
            os.remove(lcmaps_db_file1)
        if os.path.exists(lcmaps_db_file):
            os.rename(lcmaps_db_file,lcmaps_db_file1)

        if not os.path.exists(os.path.dirname(lcmaps_db_file)):
            os.makedirs(os.path.dirname(lcmaps_db_file))

        file = open(lcmaps_db_file, 'w')
        file.write('path = %s/lib/modules\n' % os.environ['GLITE_LOCATION'] ) 
        file.write('posix_enf = "lcmaps_posix_enf.mod"\n')
        file.write('" -maxuid 1"\n')
        file.write('" -maxpgid 1"\n')
        file.write('" -maxsgid 32"\n')
        file.write(' \n')
        file.write('localaccount = "lcmaps_localaccount.mod"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
        file.write(' \n')
        file.write('poolaccount = "lcmaps_poolaccount.mod"\n')
        file.write('" -override_inconsistency"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
        file.write('" -gridmapdir %s"\n'% params['host.gridmap.dir'])
        file.write(' \n')
        file.write('vomsextract = "lcmaps_voms.mod"\n')
        file.write('" -vomsdir %s"\n' % os.environ['X509_VOMS_DIR'])
        file.write('" -certdir %s"\n' % params['ca.certificates.dir'])
        file.write('\n')
        file.write('vomslocalgroup = "lcmaps_voms_localgroup.mod"\n')
        file.write('" -groupmapfile %s"\n' % params['host.groupmapfile'])
        file.write('" -mapmin 1"\n')
        file.write('\n')
        file.write('vomslocalaccount = "lcmaps_voms_localaccount.mod"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'])
        file.write('" -use_voms_gid"\n')
        file.write('\n')
        file.write('vomspoolaccount = "lcmaps_voms_poolaccount.mod"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'])
        file.write('" -gridmapdir %s"\n' % params['host.gridmap.dir'])
        file.write('" -override_inconsistency"\n')
        if params['ce.gridftp.enable']=='true':
            file.write('good             = "lcmaps_plugin_example.mod"\n')
        file.write('# policies\n')
        file.write('voms:\n')
        file.write('vomsextract -> vomslocalgroup\n')
        file.write('vomslocalgroup -> vomspoolaccount\n')
        file.write('vomspoolaccount -> posix_enf | vomslocalaccount\n')
        file.write('vomslocalaccount -> posix_enf\n')
        file.write('\n')
        file.write('standard:\n')
        file.write('localaccount -> posix_enf | poolaccount\n')
        file.write('poolaccount -> posix_enf\n')
        if params['ce.gridftp.enable']=='true':
            file.write('voms_acq:\n')
            file.write('vomsextract -> vomslocalgroup\n')
            file.write('vomslocalgroup -> vomspoolaccount\n')
            file.write('vomspoolaccount -> good | vomslocalaccount\n')

            file.write('standard_acq:\n')
            file.write('poolaccount -> good | localaccount\n')
            file.write('localaccount -> good\n')

            file.write('voms_enf:\n')
            file.write('vomsextract -> vomslocalgroup\n')
            file.write('vomslocalgroup -> vomspoolaccount\n')
            file.write('vomspoolaccount -> posix_enf | vomslocalaccount\n')
            file.write('vomslocalaccount -> posix_enf\n')

            file.write('standard_enf:\n')
            file.write('poolaccount -> posix_enf | localaccount\n')
            file.write('localaccount -> posix_enf\n')
            
        file.close()
        
        if not os.path.exists(lcmaps_db_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % lcmaps_db_file)
            return 1
    
        if self.verbose:
            print "Creating the LCMAPS DB file %s              " % lcmaps_db_file,
            glib.printOkMessage()
     
        #-------------------------------------------------------------------------------
        # Globus configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring Globus"
    
        # Configure Globus
        if globus.postInstall() != 0:
            print "Configuring Globus                                           ",
            glib.printFailedMessage()
            return 1

        print "Configuring Globus                                           ",
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Modify condor-c config file
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring Condor-C..."
    
        # Configure Condor C
        if condor.configureCondorC() != 0:
            print "Configuring Condor-C                                         ",
            glib.printFailedMessage()
            return 1
    
        # Set environment for the Condor user
        glib.setUserEnv(params['condor.user'])
        
        print "Configuring Condor-C                                             ",
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # LRMS Log Parser configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the BLAHP log parser..."
            
        # Set default values
        if not params.has_key('use.log.parser'):
            params['use.log.parser'] = "false"
        else:
            if params['use.log.parser'] != "true":
                params['use.log.parser'] = "false"
    
        # Update script files
        for key in ["%s/bin/pbs_submit.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/pbs_status.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/lsf_submit.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/lsf_status.sh" % os.environ['GLITE_LOCATION']]:

            if self.verbose:
                print "Reconfiguring %s script" % key
                
            if params['use.log.parser'] == "false":

                inFile = open(key,'r')
                outFile = open(key + ".new",'w')
                
                for line in inFile:
                    if line.strip().startswith("BLParser"):
                        outFile.write('BLParser=""\n')
                    elif line.strip().startswith("BLPserver"):
                        outFile.write('BLPserver="127.0.0.1"\n')
                    elif line.strip().startswith("BLPport"):
                        outFile.write('BLPport=""\n')
                    else:
                        outFile.write(line)
                        
                outFile.close()
                inFile.close()
                
                os.rename(key + ".new",key)
    
            else:
    
                inFile = open(key,'r')
                outFile = open(key + ".new",'w')
                
                for line in inFile:
                    if line.strip().startswith("BLParser"):
                        outFile.write('BLParser="yes"\n')
                    elif line.strip().startswith("BLPserver"):
                        outFile.write('BLPserver="%s"\n' % params['log.parser.address'])
                    elif line.strip().startswith("BLPport"):
                        if params['cemon.lrms'] == "pbs" and key.find('pbs') > -1:
                            outFile.write('BLPport="%s"\n' % params['pbs.log.parser.port'])
                        elif params['cemon.lrms'] == "lsf" and key.find('lsf') > -1:
                            outFile.write('BLPport="%s"\n' % params['lsf.log.parser.port'])
                    else:
                        outFile.write(line)
                        
                outFile.close()
                inFile.close()
                
                os.rename(key + ".new",key)

        print "Configuring the BLAHP log parser                             ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Logger configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the Logging and Bookkeeping local logger daemon..."
    
        uid,gid = glib.get_user_info(os.environ['GLITE_USER'])
        
        # Check certificate directory, create if not existing
        if self.verbose:
            print "Checking user certificates directory %s/%s" % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path'])
        glib.check_dir("%s/%s" % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']),0755, uid, gid)
        
        # Copy certificates
        if self.verbose:
            print "Copying host certificates to logger user certificates directory"
        os.system("cp %s %s ~%s/%s" % (params['host.certificate.file'], params['host.key.file'], \
            os.environ['GLITE_USER'], params['user.certificate.path']))
        os.chown("%s/%s/%s" % ( \
            pwd.getpwnam(os.environ['GLITE_USER'])[5], \
            params['user.certificate.path'], \
            os.path.basename(params['host.certificate.file'])), uid, gid)
        os.chown("%s/%s/%s" % ( \
            pwd.getpwnam(os.environ['GLITE_USER'])[5], \
            params['user.certificate.path'], \
            os.path.basename(params['host.key.file'])), uid, gid)

        # Set files mode
        os.chmod("%s/bin/pbs_submit.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/pbs_status.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/pbs_cancel.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_submit.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_status.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_cancel.sh" % os.environ['GLITE_LOCATION'],0755)
        
        print "Configuring the Logging and Bookkeeping local logger daemon  ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Configure the CE Monitor
        #-------------------------------------------------------------------------------
        
        tomcat.stop()
        
        if self.verbose:
            print "Deploying the CE Monitor web service and plug-ins..."
        
        if os.path.exists("%s/webapps/ce-monitor.war" % os.environ['CATALINA_HOME']):
            if self.verbose:
                print "Removing existing CE monitor war file..."
            os.remove("%s/webapps/ce-monitor.war" % os.environ['CATALINA_HOME'])
            
        if os.path.exists("%s/webapps/ce-monitor" % os.environ['CATALINA_HOME']):
            if self.verbose:
                print "Removing existing CE monitor files..."
            os.system("rm -rf %s/webapps/ce-monitor" % os.environ['CATALINA_HOME'])

        os.system("cp %s/etc/glite-ce-monitor/ce-monitor.xml %s/conf/Catalina/localhost" \
            % (os.environ["GLITE_LOCATION"], os.environ["CATALINA_HOME"]))
            
        # Check war file
        if not params.has_key('ce-monitor.DOCBASE'):
            params['ce-monitor.DOCBASE'] = "%s/share/webapps/ce-monitor.war" % os.environ['GLITE_LOCATION']

        if not os.path.exists(params['ce-monitor.DOCBASE']):
            glib.printErrorMessage("The CE Monitor war file does not exist")
            glib.printErrorMessage("Configuration cannot continue")
            print "Deploying the CE Monitor web service and plug-ins         ",
            glib.printFailedMessage()
            return 1

        # Customize context file
        inputfile = '%s/conf/Catalina/localhost/ce-monitor.xml' % os.environ['CATALINA_HOME']
        doc = xml.dom.minidom.parse(inputfile)
        nodes = xpath.Evaluate("/Context", doc.documentElement)
        nodes[0].setAttribute('docBase',params['ce-monitor.DOCBASE'])
        nodes = xpath.Evaluate('/Context/Environment[@name="AuthZConfFileName"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/etc/glite-ce-monitor/authconfig.xml" % os.environ['GLITE_LOCATION'])
        nodes = xpath.Evaluate('/Context/Environment[@name="sslCertFile"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/%s/hostcert.pem" % (pwd.getpwnam("tomcat4")[5], \
            params['user.certificate.path']))
        nodes = xpath.Evaluate('/Context/Environment[@name="sslKey"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/%s/hostkey.pem" % (pwd.getpwnam("tomcat4")[5], \
            params['user.certificate.path']))
        nodes = xpath.Evaluate('/Context/Logger[@className="org.apache.catalina.logger.FileLogger"]', doc.documentElement)
        nodes[0].setAttribute('prefix',"glite-ce-monitor-")
        
        try:        
            outputfile = '%s/conf/Catalina/localhost/ce-monitor.xml' % os.environ['CATALINA_HOME']
            output = open(outputfile, "w")
            PrettyPrint(doc, stream=output)
            output.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printErrorMessage("File is not writable")
            print "Deploying the CE Monitor web service and plug-ins         ",
            glib.printFailedMessage()
                
        doc.unlink()
        
        # Change owner to tomcat5 user
        uid,gid = glib.get_user_info('tomcat4')
        os.system("/bin/chown -R %s:%s %s/conf/Catalina/localhost/ce-monitor.xml" % (uid,gid,os.environ['CATALINA_HOME']))
        
        # Give tomcat a chance to expand the war file
        tomcat.start()
        print "Waiting for Tomcat to expand the war file..."
        while not os.path.exists("%s/webapps/ce-monitor/config/sensorConfig.xml" % os.environ['CATALINA_HOME']):
            time.sleep(1)
        tomcat.stop()
        
        os.symlink("%s/share/java/glite-ce-ce-plugin.jar" % os.environ['GLITE_LOCATION'], \
                "%s/webapps/ce-monitor/sensors/CESensor.jar" % os.environ['CATALINA_HOME'])

        print "\nDeploying the CE Monitor web service and plug-ins          ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Configure the CE Plugin
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the CE subscription file..."

        # tomcat.restart()
        
        # Creating the predefined subscription file
        # Subscription expiration time is about six months from now, this is reset everytime the script is run
        expTime = time.strftime("%Y-%m-%dT%H:%M:%SZ",time.localtime(time.time() + 182*24*60*60))
        #subsfile = "%s/webapps/ce-monitor/subscriptions/predefinedSubscriptionList.xml" \
        #    % os.environ['CATALINA_HOME']
        subsfile = "%s/etc/glite-ce-monitor/predefinedSubscriptionList.xml" \
            % os.environ['GLITE_LOCATION']
        if self.verbose:
            print "Creating the predefined subscriptions file %s..." % subsfile
        file = open(subsfile,'w')
        file.write('<?xml version="1.0" encoding="utf-8"?>\n')
        file.write('<ns1:subscriptions xmlns:ns1="http://glite.org/ce/monitorapij/types">\n')
        file.write('\n')
        if params.has_key('cemon.wms.host') and params.has_key('cemon.wms.port'):
            
            if len(params['cemon.wms.host']) != len(params['cemon.wms.port']):
                glib.printErrorMessage("The 'cemon.wms.host' and 'cemon.wms.port' parameters must have the same size")
                glib.printErrorMessage("Configuration cannot continue")
                return 1
                
            for idx in xrange(len(params['cemon.wms.host'])):
                if params['cemon.wms.host'][idx].strip() != "":
                    file.write('    <ns1:subscription xmlns:ns1="http://glite.org/ce/monitorapij/types" retrycount="-1">\n')
                    file.write('        <ns1:monitorConsumerURL>http://%s:%s</ns1:monitorConsumerURL>\n' % (params['cemon.wms.host'][idx], params['cemon.wms.port'][idx]))
                    file.write('        <ns1:topic>\n')
                    file.write('            <ns1:name>CE_MONITOR</ns1:name>\n')
                    file.write('            <ns1:dialect>\n')
                    file.write('                <ns1:name>ISM_CLASSAD</ns1:name>\n')
                    file.write('            </ns1:dialect>\n')
                    file.write('        </ns1:topic>\n')
                    file.write('        <ns1:policy>\n')
                    file.write('            <ns1:rate>50000</ns1:rate>\n')
                    file.write('            <ns1:query>\n')
                    file.write('                <ns1:expression><![CDATA[%s]]></ns1:expression>\n' % params['notifications.condition'])
                    file.write('                <ns1:queryLanguage>ClassAd</ns1:queryLanguage>\n')
                    file.write('            </ns1:query>\n')
                    file.write('            <ns1:action>\n')
                    file.write('                <ns1:doActionWhenQueryIs>true</ns1:doActionWhenQueryIs>\n')
                    file.write('                <ns1:name>SendNotification</ns1:name>\n')
                    file.write('            </ns1:action>\n')
                    file.write('            <ns1:action>\n')
                    file.write('                <ns1:doActionWhenQueryIs>false</ns1:doActionWhenQueryIs>\n')
                    file.write('                <ns1:name>SendExpiredNotification</ns1:name>\n')
                    file.write('            </ns1:action>\n')
                    file.write('        </ns1:policy>\n')
                    file.write('        <ns1:expirationTime>%s</ns1:expirationTime>\n' % expTime)
                    file.write('    </ns1:subscription>\n')
                    file.write('\n')
        file.write('</ns1:subscriptions>\n')
        file.close()

        # Update wsdd file
        inputfile = '%s/webapps/ce-monitor/WEB-INF/server-config.wsdd' % os.environ['CATALINA_HOME']
        doc = xml.dom.minidom.parse(inputfile)
        
        nodes = xpath.Evaluate('//service[@name="CEMonitor"]', doc)
        if len(nodes) == 0:
            glib.printErrorMessage("Invalid service description file at %s" % inputfile)
            glib.printErrorMessage("Cannot continue")
            return 1
            
        newParam = doc.createElement('parameter')
        newParam.setAttribute('name','predefinedSubscriptionListPath')
        newParam.setAttribute('value',"file://" + subsfile)
        nodes[0].appendChild(newParam)

        outputfile = inputfile
        output = open(outputfile, "w")
        PrettyPrint(doc, stream=output)
        output.close()
        
        doc.unlink

        # Generate the CE LCG Info Provider configuration file
        self.generateCEInfoProvidersFile()
        
        print "Configuring the CE subscription file                        ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Modify iptables
        #-------------------------------------------------------------------------------
    
        if self.iptables_config:
            
            if params.has_key('iptables.chain'):

                if self.verbose:
                    print "Opening required firewall ports..."
            
                if params['iptables.chain'] == "":
                    error_level = 2
                    glib.printWarningMessage("IPTables Chain not set, skipping firewall configuration")
                else:
                    
                    # Check if chain exists
                    if os.system("/sbin/iptables -L %s" % params['iptables.chain']) != 0:
                        if self.verbose:
                            print "Chain %s does not exist. Creating it..." % params['iptables.chain']
                        if os.system("/sbin/iptables -N %s" % params['iptables.chain']):
                            glib.printWarningMessage("An error occurred creating the new chain. Please configure the firewall manually")
                            error_level = 2
                        else:
                            if self.verbose:
                                print "Created chain %s" % params['iptables.chain']
                    else:
                        if self.verbose:
                            print "Using existing chain %s..." % params['iptables.chain']

                    # Configure the rules
                    rules = ["1 -m state --state NEW -m tcp -p tcp --dport 2119 -j ACCEPT",
                             "2 -m state --state NEW -m tcp -p tcp --dport 3878 -j ACCEPT",
                             "3 -m state --state NEW -m tcp -p tcp --dport 3879 -j ACCEPT",
                             "4 -m state --state NEW -m udp -p udp --dport 3879 -j ACCEPT",
                             "5 -m state --state NEW -m tcp -p tcp --dport 3882 -j ACCEPT",
                             "6 -m state --state NEW -m udp -p udp --dport 1020 -j ACCEPT",
                             "7 -m state --state NEW -m udp -p udp --dport 1021 -j ACCEPT",
                             "8 -m state --state NEW -m udp -p udp --dport 1022 -j ACCEPT",
                             "9 -m state --state NEW -m udp -p udp --dport 1023 -j ACCEPT",
                             "10 -m state --state NEW -m tcp -p tcp --dport 32768:65535 -j ACCEPT",
                             "11 -m state --state NEW -m udp -p udp --dport 32768:65535 -j ACCEPT",
                             "12 -m state --state NEW -m tcp -p tcp --dport 8443 -j ACCEPT"]
                    del_rules = ["ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:2119",
                                  "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3878",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3879",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:3879",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3882",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1020",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1021",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1022",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1023",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpts:32768:65535",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpts:32768:65535",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:8443"]

                    for idx in xrange(len(rules)):
                        # Delete rule if it exists
                        f = os.popen("/sbin/iptables -L %s" % params['iptables.chain'])
                        for line in f.xreadlines():
                            if line.find(del_rules[idx]) != -1:
                                if self.verbose:
                                    print "Rule '%s' already exist" % rules[idx]
                                break
                        else:
                            if os.system("/sbin/iptables -I %s %s" % (params['iptables.chain'],rules[idx])):
                                glib.printWarningMessage("An error occurred configuring a chain rule. Please configure the firewall manually")
                                error_level = 2
                                break
                            else:
                                if self.verbose:
                                    print "Added rule '%s'" % rules[idx]

                    if os.system('/etc/rc.d/init.d/iptables save'):
                        glib.printWarningMessage("An error occurred saving the iptables chain. Please configure the firewall manually")
                        error_level = 2
                
                print "Opening required firewall ports                              ",
                glib.printOkMessage()
        #-------------------------------------------------------------------
        # DGAS client: configure the DGAS client
        #-------------------------------------------------------------------

        if self.verbose:
            print "Configuring DGAS client ..."
        
        if params['dgas.client.enabled'] == "true":        
            errorcode = os.system("%s/etc/config/scripts/glite-dgas-client-config.py --subservice --configure" %os.environ['GLITE_LOCATION'])
            if errorcode:
                print "Configuring DGAS client                                   ",                                                
                glib.printFailedMessage()
                return 1
            else:
                print "Configuring DGAS client                                   ",
                glib.printOkMessage()             

        #-------------------------------------------------------------------
        # R-GMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if params['rgma.servicetool.activate'] == "true":
    
            if self.verbose:
                print "Configuring the R-GMA Service Publisher... "
    
            # Instantiate the rgma-servicetool class
            rgmaServicetool = gliteRgmaServicetool()
            rgmaServicetool.verbose = self.verbose
            
            # Create CE Monitor instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['cemon.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['cemon.serviceName'])
            servicetoolInstance.setServiceType(params['cemon.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['cemon.statusScript'])
            servicetoolInstance.setEndpoint(params['cemon.endpoint'])
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create Local Logger instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['locallogger.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['locallogger.serviceName'])
            servicetoolInstance.setServiceType(params['locallogger.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['locallogger.statusScript'])
            servicetoolInstance.setEndpoint("not available")
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Create Gatekeeper instance
            serviceId = "%s_%s" % (glib.fq_hostname, params['gatekeeper.serviceType'])
            servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
            
            # set params
            servicetoolInstance.setServiceName(params['gatekeeper.serviceName'])
            servicetoolInstance.setServiceType(params['gatekeeper.serviceType'])
            servicetoolInstance.setServiceVersion(self.version)
            servicetoolInstance.setStatusScript(params['gatekeeper.statusScript'])
            servicetoolInstance.setEndpoint("not available")
            
            # add instance to the gLite configuration
            if servicetoolInstance.add() == 1:
                return 1
            
            # Configure servicetool
            if rgmaServicetool.configure(glib):
                # error in configuring servicetool
                return 1
            
            print "Configuring the R-GMA Service Publisher                      ",
            glib.printOkMessage()
        
        #-------------------------------------------------------------------
        # R-GMA gin: configure R-GMA gadget-in tool
        #-------------------------------------------------------------------

        if self.verbose:
            print "Configuring R-GMA GIn ..."
        
        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --subservice --configure --cemon=%s" \
            % (os.environ['GLITE_LOCATION'],params['rgma.gin.run_ce_provider']))
        if errorcode:
            print "Configuring R-GMA GadgetIn                                   ",                                                
            glib.printFailedMessage()
            return 1
        else:
            print "Configuring R-GMA GadgetIn                                   ",
            glib.printOkMessage()        
    
        return error_level
    
    #-------------------------------------------------------------------------------
    # Generate the CE LCG Info provider config files
    #-------------------------------------------------------------------------------

    def generateCEInfoProvidersFile(self):

        # -----------------------------------------
        # Set default parameters
        # -----------------------------------------

        staticDir = "%s/etc/glite-ce-ce-plugin" % os.environ['GLITE_LOCATION']
        
        # Set queue type
        if not params.has_key('cemon.cetype'):
            params['cemon.cetype'] = "blah"
        else:
            if params['cemon.cetype'] == "":
                params['cemon.cetype'] = "blah"

        # Set CE type
        if params['cemon.cetype'] == "condorc":
            params['cemon.cetype'] = "condor"
             
        # determine memorysize
        proclist = os.popen("free | /bin/grep ^Mem | /bin/awk '{print int($2/1024)}'")
        proc = proclist.readlines()
        for item in proc:
            memorysize = item.strip()
        proclist.close()

        # Get queue names an SE from CESEBindings
        queues = []
        seIds = []
        entrypoints = []
        for bind in params['cemon.cesebinds']:
            if bind != "":
                if bind.split()[0].strip().find("|") != -1:
                    queues.append(bind.split()[0].strip().split("|"))
                else:
                    queues.append(bind.split()[0].strip())
                seIds.append(bind.split()[1].strip())
                entrypoints.append(bind.split()[2].strip())
        
        # -----------------------------------------
        # Create static files
        # -----------------------------------------

        # Creating the static CE file
        ceparamsfile = "%s/lcg-info-static-ce.conf" % staticDir
        ceparamsfile1 = "%s/lcg-info-static-ce.conf.1"  % staticDir
        ceparamsfile_tmp = "%s/lcg-info-static-ce.conf.tmp"  % staticDir
        
        if os.path.exists(ceparamsfile1):
            os.remove(ceparamsfile1)
        if os.path.exists(ceparamsfile_tmp):
            os.remove(ceparamsfile_tmp)
        if os.path.exists(ceparamsfile):
            if self.verbose:
                print "Backing up existing file to %s..." % ceparamsfile1
            os.system("cp %s %s" % (ceparamsfile,ceparamsfile1))
 
        # Create file
        if self.verbose:
            print "Creating static CE info provider configuration file %s..." % ceparamsfile
        f = open(ceparamsfile_tmp, 'w')

        for queue in params['cemon.queues']:
            f.write("dn: GlueCEUniqueID=%s:%s/%s-%s-%s\n" \
                % (glib.fq_hostname,\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                queue))
            f.write("GlueCEHostingCluster: %s\n" % params['cemon.cluster'])
            f.write("GlueCEName: %s\n" % queue)
            f.write("GlueCEInfoGatekeeperPort: %s\n" % params['gatekeeper.port'])
            f.write("GlueCEInfoHostName: %s\n" % glib.fq_hostname)
            f.write("GlueCEInfoLRMSType: %s\n" % params['cemon.lrms'])
            f.write("GlueCEInfoLRMSVersion: %s\n" % params['cemon.lrms.version'])
            f.write("GlueCEInfoTotalCPUs: 0\n")
            f.write("GlueCEInfoJobManager: %s\n" % params['cemon.lrms'])
            f.write("GlueCEInfoContactString: %s:%s/%s-%s-%s\n" \
                % (glib.fq_hostname,\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                queue))
            f.write("GlueCEInfoApplicationDir: %s\n" % params['vo.sw.dir'])
            f.write("GlueCEInfoDataDir: %s\n" % params['ce.datadir'])
            if len(seIds) > 0:
                f.write("GlueCEInfoDefaultSE: %s\n" % seIds[0])
            f.write("GlueCEStateEstimatedResponseTime: 999999\n")
            f.write("GlueCEStateFreeCPUs: 0\n")
            f.write("GlueCEStateRunningJobs: 0\n")
            f.write("GlueCEStateStatus: Production\n")
            f.write("GlueCEStateTotalJobs: 0\n")
            f.write("GlueCEStateWaitingJobs: 0\n")
            f.write("GlueCEStateWorstResponseTime: 0\n")
            f.write("GlueCEStateFreeJobSlots: 0\n")
            f.write("GlueCEPolicyMaxCPUTime: 999999\n")
            f.write("GlueCEPolicyMaxRunningJobs: 999999\n")
            f.write("GlueCEPolicyMaxTotalJobs: 999999\n")
            f.write("GlueCEPolicyMaxWallClockTime: 999999\n")
            f.write("GlueCEPolicyPriority: 1\n")
            f.write("GlueCEPolicyAssignedJobSlots: 0\n")

            voQueue = "false"
            queueacls=""
            for vo in glib.getVoList():
                if glib.getVoParams(vo).has_key('cemon.queues'):
                    voQueue = "true"
                    voCemonQueues = glib.getVoParams(vo)['cemon.queues']
                    #print "Analyzing queue %s vo %s voQueues %s" %(queue,vo,voCemonQueues)

                    if isinstance(voCemonQueues,list):
                       if queue in voCemonQueues:
                          queueacls += "GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name']
                          f.write("GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name'])
                    else:
                        if queue in (voCemonQueues,):
                            queueacls += "GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name']  
                            f.write("GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name'])
            if voQueue == "false":
                queueacls = self.getQueueACL(queue)
                if queueacls == 1:
                    glib.printWarningMessage("Could not contact the queue manager")
                    glib.printWarningMessage("and no Torque Server configuration is available on this node")
                    glib.printWarningMessage("Queues ACLs have not been published")
                elif queueacls != "":
                    f.write(queueacls)
            print "queueacls for queue %s %s" %(queue,queueacls)

#            queueacls = self.getQueueACL(queue)
#            if queueacls == 1:
#                glib.printWarningMessage("Could not contact the queue manager")
#                glib.printWarningMessage("and no Torque Server configuration is available on this node")
#                glib.printWarningMessage("Queues ACLs have not been published")
#            elif queueacls != "":
#                f.write(queueacls)

            f.write("GlueForeignKey: GlueClusterUniqueID=%s\n" % params['cemon.cluster'])
            f.write("GlueInformationServiceURL: ldap://%s:2170/mds-vo-name=resource,o=grid\n" % glib.fq_hostname)

            f.write("\n")
            
            voviews = queueacls.split('GlueCEAccessControlBaseRule: VO:')
            for voview in voviews:
                vo = voview.strip()
                if vo != "":
                    voParam = glib.getVoParams(vo)
                    if params['ce.datadir'].strip() == '':
                        params['ce.datadir'] = 'unset'
                    f.write("dn: GlueVOViewLocalID=%s,GlueCEUniqueID=%s:%s/%s-%s-%s\n" \
                    % ( vo, \
                        glib.fq_hostname, \
                        params['gatekeeper.port'], \
                        params['cemon.cetype'], \
                        params['cemon.lrms'], \
                        queue))
                    f.write("GlueCEAccessControlBaseRule: VO:%s\n" % vo)
                    f.write("GlueCEStateRunningJobs: 0\n")
                    f.write("GlueCEStateWaitingJobs: 0\n")
                    f.write("GlueCEStateTotalJobs: 0\n")
                    f.write("GlueCEStateFreeJobSlots: 0\n")
                    f.write("GlueCEStateEstimatedResponseTime: 0\n")
                    f.write("GlueCEStateWorstResponseTime: 0\n")
                    if len(seIds) > 0:
                        f.write("GlueCEInfoDefaultSE: %s\n" % seIds[0])
                    f.write("GlueCEInfoApplicationDir: %s\n" % voParam['SW_DIR'])
                    f.write("GlueCEInfoDataDir: %s\n" % params['ce.datadir'])
                    f.write("GlueChunkKey: GlueCEUniqueID=%s:%s/%s-%s-%s\n" \
                    % ( glib.fq_hostname,\
                        params['gatekeeper.port'],\
                        params['cemon.cetype'],\
                        params['cemon.lrms'],\
                        queue))
                        
                    f.write("\n")

        f.close()
        
        os.rename(ceparamsfile_tmp,ceparamsfile)
         
        os.system("%s/sbin/lcg-info-static-create -c %s/lcg-info-static-ce.conf -t %s/etc/GlueCE.template > %s/var/gip/ldif/lcg-info-static-ce.ldif" \
                   % (params['LCG_LOCATION'], \
                      staticDir, \
                      params['LCG_LOCATION'], \
                      params['LCG_LOCATION']))
        
        # Creating the static cluster file
        ceparamsfile = "%s/lcg-info-static-cluster.conf" % staticDir
        ceparamsfile1 = "%s/lcg-info-static-cluster.conf.1"  % staticDir
        ceparamsfile_tmp = "%s/lcg-info-static-cluster.conf.tmp"  % staticDir
        
        if os.path.exists(ceparamsfile1):
            os.remove(ceparamsfile1)
        if os.path.exists(ceparamsfile_tmp):
            os.remove(ceparamsfile_tmp)
        if os.path.exists(ceparamsfile):
            if self.verbose:
                print "Backing up existing file to %s..." % ceparamsfile1
            os.system("cp %s %s" % (ceparamsfile,ceparamsfile1))
 
        # Create file
        if self.verbose:
            print "Creating static Cluster info provider configuration file %s..." % ceparamsfile
        f = open(ceparamsfile_tmp, 'w')

        f.write("dn: GlueClusterUniqueID=%s\n" % params['cemon.cluster'])
        f.write("GlueClusterName: %s\n" % params['cemon.cluster'])
        f.write("GlueClusterService: %s\n" % params['cemon.cluster'])
        for queue in params['cemon.queues']:
            f.write("GlueForeignKey: GlueCEUniqueID=%s:%s/%s-%s-%s\n" \
                % (os.environ['HOSTNAME'],\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                queue))
        f.write("GlueInformationServiceURL: ldap://%s:2170/mds-vo-name=resource,o=grid\n" % glib.fq_hostname)

        f.write("\n")
        
        f.write("dn: GlueSubClusterUniqueID=%s, GlueClusterUniqueID=%s\n" \
            % (params['cemon.cluster'],params['cemon.cluster']))

        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: GLITE_3_0_0\n")
        for item in params['custom.runtime.environment']:
            if item.strip() != "":
                f.write("GlueHostApplicationSoftwareRunTimeEnvironment: %s\n" % item)

        f.write("GlueHostArchitectureSMPSize: %s\n" % params['ldif.ce_smpsize'])    #    CE_SMPSIZE
        f.write("GlueHostBenchmarkSF00: %s\n" % params['ldif.ce_sf00'])        #    CE_SF00
        f.write("GlueHostBenchmarkSI00: %s\n" % params['ldif.ce_si00'])        #    CE_SI00
        f.write("GlueHostMainMemoryRAMSize: %s\n" % memorysize) #    CE_MINPHYSMEM
        f.write("GlueHostMainMemoryVirtualSize: %s\n" % (int(memorysize)*2))    #    CE_MINVIRTMEM
        f.write("GlueHostNetworkAdapterInboundIP: %s\n" % params['ldif.ce_inboundip'])        #    CE_INBOUNDIP
        f.write("GlueHostNetworkAdapterOutboundIP: %s\n" % params['ldif.ce_outboundip'])        #    CE_OUTBOUNDIP
        f.write("GlueHostOperatingSystemName: %s\n" % params['ldif.ce_os'])    #    CE_OS
        f.write("GlueHostOperatingSystemRelease: %s\n" % params['ldif.ce_os_release'])    #    CE_OS_RELEASE
        f.write("GlueHostOperatingSystemVersion: %s\n" % params['ldif.ce_os_version'])    #    CE_OS_VERSION
        f.write("GlueHostProcessorClockSpeed: %s\n" % params['ldif.ce_cpu_speed'])    #    CE_CPU_SPEED
        f.write("GlueHostProcessorModel: %s\n" % params['ldif.ce_cpu_model'])           #    CE_CPU_MODEL
        f.write("GlueHostProcessorVendor: %s\n" % params['ldif.ce_cpu_vendor'])       #    CE_CPU_VENDOR
        f.write("GlueSubClusterName: %s\n" % params['cemon.cluster'])
        f.write("GlueSubClusterPhysicalCPUs: 0\n")
        f.write("GlueSubClusterLogicalCPUs: 0\n")
        f.write("GlueSubClusterTmpDir: /tmp\n")
        f.write("GlueSubClusterWNTmpDir: /tmp\n")
        f.write("GlueInformationServiceURL: ldap://%s:2170/mds-vo-name=resource,o=grid\n" % glib.fq_hostname)

        f.write("GlueChunkKey: GlueClusterUniqueID=%s\n" % params['cemon.cluster'])

        f.write("\n")

        f.write("dn: GlueLocationLocalID=GLITE_3_0_0,GlueSubClusterUniqueID=%s,GlueClusterUniqueID=%s\n" \
                 % (params['cemon.cluster'],params['cemon.cluster']))
        f.write("GlueLocationName: gLite\n")
        f.write("GlueLocationVersion: 3.0\n")
        f.write("GlueLocationPath: %s\n" % os.environ['GLITE_LOCATION'])
        f.write("GlueChunkKey: GlueSubClusterUniqueID=%s\n" % params['cemon.cluster'])

        f.close()
        
        os.rename(ceparamsfile_tmp,ceparamsfile)
         
        os.system("%s/sbin/lcg-info-static-create -c %s/lcg-info-static-cluster.conf -t %s/etc/GlueCluster.template > %s/var/gip/ldif/lcg-info-static-cluster.ldif" \
                   % (params['LCG_LOCATION'], \
                      staticDir, \
                      params['LCG_LOCATION'], \
                      params['LCG_LOCATION']))
        
        # Creating the static cesebindings file
        ceparamsfile = "%s/lcg-info-static-cesebind.conf" % staticDir
        ceparamsfile1 = "%s/lcg-info-static-cesebind.conf.1"  % staticDir
        ceparamsfile_tmp = "%s/lcg-info-static-cesebind.conf.tmp"  % staticDir
        
        if os.path.exists(ceparamsfile1):
            os.remove(ceparamsfile1)
        if os.path.exists(ceparamsfile_tmp):
            os.remove(ceparamsfile_tmp)
        if os.path.exists(ceparamsfile):
            if self.verbose:
                print "Backing up existing file to %s..." % ceparamsfile1
            os.system("cp %s %s" % (ceparamsfile,ceparamsfile1))
 
        # Create file
        if self.verbose:
            print "Creating static CE-SE Bindings info provider configuration file %s..." % ceparamsfile
        f = open(ceparamsfile_tmp, 'w')

        for q in params['cemon.queues']:
            f.write("dn: GlueCESEBindGroupCEUniqueID=%s:%s/%s-%s-%s\n" \
                % (glib.fq_hostname,\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                q))
            for idx in xrange(len(seIds)):
                if queues[idx] == "'.'" or queues[idx] == q:
                    f.write("GlueCESEBindGroupSEUniqueID: %s\n" % seIds[idx])
                    break
            else:
                f.write("GlueCESEBindGroupSEUniqueID: none\n")
            f.write("\n")
            
        for idxSE in xrange(len(seIds)):
            for idxq in xrange(len(params['cemon.queues'])):
                if queues[idxSE] == "'.'" or queues[idxSE] == params['cemon.queues'][idxq]:
                    f.write("dn: GlueCESEBindSEUniqueID=%s, GlueCESEBindGroupCEUniqueID=%s:%s/%s-%s-%s\n" \
                        % (seIds[idxSE],\
                        glib.fq_hostname,\
                        params['gatekeeper.port'],\
                        params['cemon.cetype'],\
                        params['cemon.lrms'],\
                        params['cemon.queues'][idxq]))
                    f.write("GlueCESEBindCEAccesspoint: %s\n" % entrypoints[idxSE])
                    f.write("GlueCESEBindCEUniqueID: %s:%s/%s-%s-%s\n" \
                        % (glib.fq_hostname,\
                        params['gatekeeper.port'],\
                        params['cemon.cetype'],\
                        params['cemon.lrms'],\
                        params['cemon.queues'][idxq]))
                    f.write("GlueCESEBindMountInfo: %s\n" % entrypoints[idxSE])
                    f.write("GlueCESEBindWeight: 0\n")

                f.write("\n")
                
        f.close()
        
        os.rename(ceparamsfile_tmp,ceparamsfile)
         
        os.system("%s/sbin/lcg-info-static-create -c %s/lcg-info-static-cesebind.conf -t %s/etc/GlueCESEBind.template > %s/var/gip/ldif/lcg-info-static-cesebind.ldif" \
                   % (params['LCG_LOCATION'], \
                      staticDir, \
                      params['LCG_LOCATION'], \
                      params['LCG_LOCATION']))
        
        plugin_dir = "%s/var/gip/plugin" % params['LCG_LOCATION']
        
        if params['cemon.lrms'] == "pbs":
            pbsServer = params['log.parser.hostname']
            f = open('%s/ce-pbs.sh' % plugin_dir, 'w')
            f.write("#!/bin/sh\nexport LANG=C\n")
            f.write("%s/libexec/lcg-info-dynamic-pbs %s/var/gip/ldif/lcg-info-static-ce.ldif %s\n" \
                % (params['LCG_LOCATION'], \
                   params['LCG_LOCATION'],
                   pbsServer))
            f.close()
            os.system("chmod +x %s/ce-pbs.sh" % plugin_dir)

        elif params['cemon.lrms'] == "lsf":
            f = open('%s/ce-lsf.sh' % plugin_dir, 'w')
            f.write("#!/bin/sh\nexport LANG=C\n")
            f.write("%s/libexec/lcg-info-dynamic-lsf %s %s/var/gip/ldif/lcg-info-static-ce.ldif\n" \
                % (params['LCG_LOCATION'], \
                    os.environ['LSF_BIN_PATH'], \
                    params['LCG_LOCATION']))
            f.close()
            os.system("chmod +x %s/ce-lsf.sh" % plugin_dir)

        elif params['cemon.lrms'] == "condor":
            f = open('%s/ce-condor.sh' % plugin_dir, 'w')
            f.write("#!/bin/sh\nexport LANG=C\n")
            f.write("%s/libexec/lcg-info-dynamic-condor /opt/condor-c/bin %s/var/gip/ldif/lcg-info-static-ce.ldif\n" \
                % (params['LCG_LOCATION'],\
                   params['LCG_LOCATION']))
            f.close()
            os.system("chmod +x %s/ce-condor.sh" % plugin_dir)
            
        #Create glite-ce-info script
        
        print "Creating glite-ce-info script                         "
        f = open("%s/etc/glite-ce-ce-plugin/glite-ce-info" % os.environ['GLITE_LOCATION'],'w')
        f.write("#!/bin/sh\nexport LANG=C\n")
        f.write("%s/bin/lcg-info-generic %s/etc/lcg-info-generic.conf\n" % (params['LCG_LOCATION'],params['LCG_LOCATION']))
        f.close()
        os.system("chmod +x %s/etc/glite-ce-ce-plugin/glite-ce-info" % os.environ['GLITE_LOCATION'])

        glib.check_dir("%s/var/gip/tmp" % params['LCG_LOCATION'],0777)

        print "Configuring CE Information providers                         ",
        glib.printOkMessage()
        
        return 0
            
    #Function to check if a certain queue has permissions set
    #It tries to get the acl groups and match them with a given VO.
    #If the batch system is torque: it uses the qmgr command, if it fails, it reads from the torque-server xml 
    #configuration file when pbs is running locally
    #If the batch system is lsf it will execute the bqueues command to get the groups information
    #Input parameters: queueName to get the acl information from
    #Return parameters: 1 in case it fails
    #                   string containing the GlueSchema syntax to define all the GlueCEAccessControlBaseRule for the queue        
    def getQueueACL(self,queueName):
        outputString = ""
        torqueServer = ""
        aclEnable = "False"
        readMethod = "qmgr"
        
        qAllowed = list()
        qDenied = list()
    
        if params['cemon.lrms'] == "lsf":
            for vo in glib.getVoList():
                command = "bqueues -u %s %s" %(glib.getVoParams(vo)['pool.account.group'],queueName)        
                fInput,fOutput,fError = os.popen3(command,'r',0)
                
                for line in fOutput.xreadlines():
                    if line != "" and line.strip().find(queueName) != -1:
                        outputString += "GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name']
                fInput.close()
                fOutput.close()
                fError.close()
            if outputString == "":
                outputString += "GlueCEAccessControlBaseRule: VO:\n"
            return outputString
        else:
            if params['cemon.lrms'] != 'pbs':
                for vo in glib.getVoList():
                    outputString += "GlueCEAccessControlBaseRule: VO:%s\n" %glib.getVoParams(vo)['vo.name']
                return outputString            

        #Check if the log server is running locally
        torqueServer = params['log.parser.hostname']
                
        #Try to get the ACL information using the qmgr command
        command = "qmgr %s -c 'print queue %s'\n" %(torqueServer,queueName)
        fInput,fOutput,fError = os.popen3(command,'r',0)
        
        if fError.readline().strip().find("Connection refused") != -1:
            glib.printWarningMessage("The pbs_server daemon is not running on %s" % torqueServer)
                
            #Try to start pbs_server if it's run locally
            if (torqueServer == glib.fq_hostname):
                glib.printInfoMessage("Trying to start the pbs_server %s locally" % torqueServer)
                if glib.startProcess('pbs_server','/etc/rc.d/init.d/pbs_server start') != 0:
                    glib.printErrorMessage("It was not possible to start the pbs_server, trying with the configuration file")
                    readMethod = "file"
                else:
                    if self.verbose:
                        print "pbs_server successfully started at %s        " % torqueServer
                        glib.printOkMessage()
            else:
                glib.printErrorMessage("It has been impossible to contact the pbs_server at %s and get the queue ACL information" % torqueServer)
                return 1
        fInput.close()
        fOutput.close()
        fError.close()        
        
        #Try to read the ACL information from the configuration file
        if readMethod == "file":
            glib.printInfoMessage("Using xml configuration file to read the queues ACLs")
            if glib.loadConfiguration("%s/../glite-torque-server.cfg.xml" % glib.getScriptPath(), params):
                glib.printErrorMessage("Please, check that the glite-torque-server.cfg.xml file is available")
                return 1
                
            instances = glib.getInstances("pbs-queue")
                                                                                                                                                                       
            queueFound = 1
            for instance in instances:
                iParams = glib.getInstanceParams(instance)
                if (iParams['queue.name'] == queueName):
                    if (iParams['queue.acl.group.enable'] == "True" and iParams['queue.acl.groups'] != ""):
                        found = 0
                        aclEnable = "True"
                        
                        #Looking for initial and ending '
                        tmpGroupACL = iParams['queue.acl.groups'].strip()
                        if (tmpGroupACL[0] == "'"):
                            tmpGroupACL = tmpGroupACL[1:]
                        
                        if (tmpGroupACL[len(tmpGroupACL) -1] == "'"):
                            tmpGroupACL = tmpGroupACL[0:-1]

                        listGroupACL = tmpGroupACL.split(',')
                        
                        for item in listGroupACL:
                            groupACL = item.strip()
                            permissionType = groupACL[0]
                            if (permissionType == '+') or (permissionType == '-'):
                                groupACL = groupACL[1:]
                            else:
                                permissionType = "+"

                            if (groupACL not in params['pool.account.group']):
                                glib.printWarningMessage("The group %s for queue %s does not appear in pool.account.group" %(groupACL,queueName))        
                            else:
                                if permissionType != '-':
                                    qAllowed.append(groupACL)
                                else:
                                    qDenied.append(groupACL)
                    break
                                    
        #Read the ACL information from the qmgr command
        if (readMethod == "qmgr"):
            if self.verbose:
                glib.printInfoMessage("Using qmgr command to read the queues ACLs, running command %s " %command)
            
            fInput,fOutput,fError = os.popen3(command,'r',0)
            #Check if the queue does exist
            if fError.readline().strip().find("Unknown queue") != -1:
                glib.printWarningMessage("The queue %s has not been found in the pbs_server %s" %(queueName,torqueServer))
                fInput.close()
                fOutput.close()
                fError.close()
            else:
                for line in fOutput.xreadlines():
                    if (not line.strip().startswith('#') and line.find('acl_group_enable') != -1):
                        aclEnable = line.split("=")[1].strip()
                
                    if (aclEnable == "True" and not line.strip().startswith('#') and line.find('acl_groups') != -1):
                        groupACL = line.split("=")[1].strip()
                        permissionType = groupACL[0]
                
                        if ((permissionType == '+') or (permissionType == '-')):
                            groupACL = groupACL[1:]
                        else:
                            permissionType = "+"
                    
                        groupFound = 0
                        for vo in glib.getVoList():
                            if glib.getVoParams(vo)['pool.account.group'].find(groupACL) != -1:
                                groupFound = 1
                                break

                        if not groupFound:
                            glib.printWarningMessage("The group %s for queue %s does not appear in pool.account.group for any defined VO" %(groupACL,queueName))                            
                        else:    
                            if (permissionType != '-'):
                                qAllowed.append(groupACL)
                            else:
                                qDenied.append(groupACL)

                fInput.close()
                fOutput.close()
                fError.close()
                
                
        #Create the Glue Schema syntax
        if (aclEnable == "True"): #Maybe there are aclgroups but aclEnable is not even defined
            #We find allowed groups, do not care about the banned ones
            if (len(qAllowed) > 0):
                for item in qAllowed:
                    voFound = 1
                    for vo in glib.getVoList():
                        if (glib.getVoParams(vo)['pool.account.group'] == item):
                            voName = glib.getVoParams(vo)['vo.name']
                            outputString += "GlueCEAccessControlBaseRule: VO:%s\n" %voName
                            voFound = 0
                    if(voFound == 1):
                        glib.printWarningMessage("It was not possible to find the VO matching the acl group %s for the queue %s" %(item,queueName))
            else:
                #There are not allowed and banned groups, though aclgroups is set to true, we deny all the VOs
                if (len(qAllowed) == 0 and len(qDenied)==0):
                   outputString = "GlueCEAccessControlBaseRule: VO:\n"
        else:
            glib.printInfoMessage("There are no ACLs permissions set for the queue %s" %queueName)
            for vo in glib.getVoList():
                voName = glib.getVoParams(vo)['vo.name']
                outputString += "GlueCEAccessControlBaseRule: VO:%s\n" %voName
        
        return outputString
        
    #-------------------------------------------------------------------------------
    # Start all services
    #-------------------------------------------------------------------------------

    def start(self):

        if len(os.listdir(params['X509_VOMS_DIR'])) == 0:
            print("\n\n")
            glib.printErrorMessage("No VOMS Server host certificates could be found in %s" % params['X509_VOMS_DIR'])
            glib.printErrorMessage("Before using the %s," % service.friendly_name)
            glib.printErrorMessage("make sure that the host certificate of each used VOMS")
            glib.printErrorMessage("Server is installed in the %s directory." % params['X509_VOMS_DIR'])
            return 1

        #-------------------------------------------------------------------------------
        # Start the gatekeeper
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'gk':

            gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']
            pid = glib.getPID('glite-gatekeeper')
        
            if pid != 0:
                glib.printInfoMessage('Gatekeeper is already running. Stopping and restarting the gLite Gatekeeper ...')
                os.system('kill -9 %d' % pid)
            else:
                print "Starting the gLite gatekeeper ... "
        
            glib.system( "%s -conf %s/etc/gatekeeper.conf" % ( gatekeeper, os.environ['GLITE_LOCATION']) )
        
            # Check that the daemon is running
            pid = glib.getPID('glite-gatekeeper')
            if pid != 0:
                print "Starting the gLite Gatekeeper                                ",                                                
                glib.printOkMessage()
            else:
                print "Starting the gLite Gatekeeper                                ",                                                
                glib.printFailedMessage()
                return 1
    
        #-------------------------------------------------------------------------------
        # Start the locallogger
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'll':
    
            locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']
    
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
            if (pid1 != 0) or (pid2 != 0):
                glib.printInfoMessage('LB Local Logger daemon already running. Stopping and restarting the LB Local Logger Daemons ...')
                os.system( "%s stop" % locallogger)
            else:
                print "Starting the gLite LB Local Logger ..."
        
            os.system( "%s start" % locallogger)
        
            # Check that the daemon is running
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
            if (pid1 != 0) and (pid2 != 0):
                print "The LB Local Logger was started                              ",                                                
                glib.printOkMessage()
                print "The LB Interlogger was started                               ",                                                
                glib.printOkMessage()
            else:
                glib.printErrorMessage("Could not start the LB Local Logger")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
    
        #-------------------------------------------------------------------------------
        # Start Tomcat
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'cemon':
    
            print "Starting Tomcat and the CE Monitor ..."
            pid = glib.getPID("tomcat")
            if pid != 0:
                glib.printInfoMessage("Tomcat is running already. Stopping and restarting Tomcat ...")
                tomcat.stop()
                
            tomcat.start()
        
            # Check that the daemon is running
            pid = glib.getPID("tomcat")
            if pid == 0:
                glib.printErrorMessage("Could not start Tomcat")
                glib.printErrorMessage("Please verify and re-run the script")
                return 1
            else:
                print "Tomcat has been started                                      ",                                                
                glib.printOkMessage()
            
        #-------------------------------------------------------------------
        # Start Dgas client
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'dgas':
    
            if params['dgas.client.enabled'] == "true":
                errorcode = os.system("%s/etc/config/scripts/glite-dgas-client-config.py --start" %os.environ['GLITE_LOCATION'])
    
                if (errorcode != 0):
                    return 1
    
        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'st':
    
            if params['rgma.servicetool.activate'] == "true":
        
                errorcode = rgmaServicetool.start()
        
                if (errorcode != 0):
                    return 1
            
        #-------------------------------------------------------------------
        # Start R-GMA gin
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'gin':
    
            print "Starting the gLite R-GMA GIN service ... "
                           
            errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --start --subservice" % os.environ['GLITE_LOCATION'])
            if errorcode:
                print "Starting the gLite R-GMA GIN service                         ",                                                
                glib.printFailedMessage()
                return 1
    
            print "Starting the gLite R-GMA GIN service                         ",                                                
            glib.printOkMessage()
            
        #---------------------------------------------------------------------
        #    Start gridftp server
        #---------------------------------------------------------------------
        if params['ce.gridftp.enable']=='true':
            command = "%s/sbin/edg-gridftpd -C %s/etc/gridftpd_lcas_lcmaps.conf -S -p 2811 -u 002 -o -a -Z %s/globus-gridftp.log" \
                %(os.environ['EDG_LOCATION'], os.environ['GLITE_LOCATION'], os.environ['GLITE_LOCATION_LOG'])
            errorcode = glib.system(command)
            if errorcode:
                print "Starting the gridftp service                         ",                                                
                glib.printFailedMessage()
                return 1
    
            print "Starting the gridftp service                         ",                                                
            glib.printOkMessage()
        #"$EDG_LOCATION/sbin/edg-gridftpd -C $GLITE_LOCATION/etc/gridftpd_lcas_lcmaps.conf -S -p 2811 -u 002 -o -a -Z $GLITE_LOCATION_LOG/globus-gridftp.log
            
        return 0
    
    #-------------------------------------------------------------------------------
    # Stop all services
    #-------------------------------------------------------------------------------

    def stop(self):

        error_level = 0
        
        #---------------------------------------------------------------------
        #    Stop gridftp server
        #---------------------------------------------------------------------
        if params['ce.gridftp.enable']=='true':
            pname = "ftpd: accepting connections on port 2811"
            proclist = os.popen('/bin/ps axww | /bin/grep "%s"' % pname)
            proc = proclist.readlines()
            for item in proc:
                if (item.find('/bin/grep %s' % pname) == -1) and \
                            (item.find('%s' % pname) != -1) and \
                            (item.find('-config.py') == -1):
                    pid = string.splitfields(string.strip(item), ' ')[0]
                    break
    
            errorcode = os.system('kill -TERM %s' % pid)
            # kill -TERM ....
        
        #-------------------------------------------------------------------
        # Stop Condor
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'condor':

            condor.stop()
    
        #-------------------------------------------------------------------------------
        # Stop the gatekeeper
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'gk':

            gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']
    
            pid = glib.getPID('glite-gatekeeper')
            if pid != 0:
                print 'Stopping the gLite Gatekeeper...'
                os.system('kill -9 %d' % pid)
                pid = glib.getPID('glite-gatekeeper')
                if pid != 0:
                    print "Stopping the gLite gatekeeper                                ",                                        
                    glib.printFailedMessage()
                    error_level = 1
                else:
                    print "Stopping the gLite gatekeeper                                ",                                        
                    glib.printOkMessage()
            else:
                print "The gLite gatekeeper was already stopped                     ",                                        
                glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Stop the local logger
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'll':

            locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']
    
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
    
            if (pid1 != 0) or (pid2 != 0):
                print 'Stopping the gLite LB Local Logger ...'
                os.system( "%s stop" % locallogger)
                time.sleep(5)
                pid1 = glib.getPID('glite-lb-logd')
                pid2 = glib.getPID('glite-lb-interlogd')
                if (pid1 != 0) or (pid2 != 0):
                    print "Stopping the LB Local Logger                                 ",                            
                    glib.printFailedMessage()
                    error_level = 1
                else:
                    print "Stopping the LB Local Logger                                 ",                            
                    glib.printOkMessage()
            else:
                print "The LB Local Logger was already stopped                      ",                            
                glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Stop Tomcat
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'cemon':
    
            pid = glib.getPID("tomcat")
            if pid != 0:
                print "Stopping Tomcat and the CE Monitor ..."
                tomcat.stop()
                time.sleep(5)
                pid = glib.getPID("tomcat")
                if pid != 0:
                    print "Stopping Tomcat                                              ",                
                    glib.printFailedMessage()
                    error_level = 1
                else:
                    print "Stopping Tomcat                                              ",                
                    glib.printOkMessage()
            else:
                print "Tomcat was already stopped                                   ",                
                glib.printOkMessage()
                
        #-------------------------------------------------------------------
        # Stop the DGAS client
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'dgas':

            if params['dgas.client.enabled'] == "true":
                os.system("%s/etc/config/scripts/glite-dgas-client-config.py --stop" %os.environ['GLITE_LOCATION'])    
    
        #-------------------------------------------------------------------
        # Stop the servicetool
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'st':
    
            if params['rgma.servicetool.activate'] == "true":
        
                if rgmaServicetool.stop() != 0:
                    error_level = 1
        
        #-------------------------------------------------------------------
        # Stop R-GMA gin
        #-------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'gin':

            print "Stopping the gLite R-GMA GIN service ... "
                           
            errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --stop --subservice" % os.environ['GLITE_LOCATION'])
            if errorcode:
                print "Stopping the gLite R-GMA GIN service                         ",
                glib.printFailedMessage()
                error_level = 1
            
            print "Stopping the gLite R-GMA GIN service                         ",
            glib.printOkMessage()
                
        return error_level
        
    #-------------------------------------------------------------------------------
    # Service status
    #-------------------------------------------------------------------------------

    def status(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "       Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        error_level = 0
        
        #-------------------------------------------------------------------------------
        # Gatekeeper
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'gk':

            gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']
            pid = glib.getPID('glite-gatekeeper')
            if pid != 0:
                print "The gLite Gatekeeper is running"
            else:
                print "The gLite Gatekeeper is not running"
                error_level = 1
    
        #-------------------------------------------------------------------------------
        # Local logger
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'll':
    
            locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']
            retval = os.system( "%s status" % locallogger)
            if retval != 0:
                error_level = 1
    
        #-------------------------------------------------------------------------------
        # CE Monitor
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'cemon':
    
            if params.has_key('host.certificate.file'):
                hostcert_file = params['host.certificate.file']
            else:
                hostcert_file = "/etc/grid-security/hostcert.pem"
            
            if params.has_key('host.key.file'):    
                hostkey_file  = params['host.key.file']
            else:
                hostkey_file = "/etc/grid-security/hostkey.pem"
            
            print "The CE Monitor is ",
            status = glib.checkHTTPSconnection("localhost", 
                "/ce-monitor/services/Version?wsdl",
                8443, hostkey_file, hostcert_file)
                                        
            if status[0]:
                print "not running."
                error_level = 1
            else:
                print "running."
                    
            if error_level:
                print "\nThe %s is not running." % self.friendly_name
            else:
                print "\nThe %s is running." % self.friendly_name
                
        #-------------------------------------------------------------------------------
        # Servicetools
        #-------------------------------------------------------------------------------
    
        if self.servicecommand == 'all' or self.servicecommand == 'st':
    
            if params['rgma.servicetool.activate'] == "true":
        
                if rgmaServicetool.status():
                    error_level = 1
    
        return error_level
        
#-------------------------------------------------------------------------------
# Init params
#-------------------------------------------------------------------------------

def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['mkgridmap.script'] = "%s/sbin/glite-mkgridmap" % params['GLITE_LOCATION']
    params['mkgridmap.conf'] = "%s/etc/glite-mkgridmap.conf" % params['GLITE_LOCATION']
    params['host.gridmap.dir'] = "/etc/grid-security/gridmapdir"
    params['custom.runtime.environment'] = []
    params['dgas.client.enabled'] = 'true'
    
    params['cemon.serviceName'] = 'CE Monitor Service at %s' % glib.fq_hostname
    params['cemon.serviceType'] = 'org.glite.ce.monitor'
    params['cemon.statusScript'] = '/usr/bin/python %s/etc/config/scripts/glite-ce-config.py --status --cemon' % params['GLITE_LOCATION']
    params['cemon.endpoint'] = 'https://%s:8443/ce-monitor/services/CEMonitor' % glib.fq_hostname
    
    params['locallogger.serviceName'] = 'LB Local Logger at %s' % glib.fq_hostname
    params['locallogger.serviceType'] = 'org.glite.lb.locallogger'
    params['locallogger.statusScript'] = '%s/etc/init.d/glite-lb-locallogger status' % params['GLITE_LOCATION']
    params['locallogger.endpoint'] = 'not available'

    params['gatekeeper.serviceName'] = 'Gatekeeper at %s' % glib.fq_hostname
    params['gatekeeper.serviceType'] = 'org.glite.ce.gatekeeper'
    params['gatekeeper.statusScript'] = '%s/sbin/glite-gatekeeper -test'  % params['GLITE_LOCATION']
    params['gatekeeper.endpoint'] = 'not available'

#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():
    #Create the unique list of queues    
    if not params.has_key('cemon.queues'):
        params['cemon.queues']=list()
        for vo in glib.getVoList():
            if glib.getVoParams(vo).has_key('cemon.queues'):
                voCemonQueues = glib.getVoParams(vo)['cemon.queues']
                if isinstance(voCemonQueues,list):
                    for i in voCemonQueues:
                        if not voCemonQueues in params['cemon.queues']:
                            params['cemon.queues'].append(i)
                else:
                    if not voCemonQueues in params['cemon.queues']:
                        params['cemon.queues'].append(voCemonQueues)

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('CONDOR_CONFIG')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('EDG_LOCATION')
    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')

    if params['cemon.lrms'] in ['pbs','torque']:
        glib.export('PBS_BIN_PATH',params['cemon.cluster-batch-system-bin-path'])
        glib.export('PBS_SPOOL_DIR',params['PBS_SPOOL_DIR'])
    elif params['cemon.lrms'] == 'lsf':
        glib.export('LSF_BIN_PATH',params['cemon.cluster-batch-system-bin-path'])
        glib.export('LSF_CONF_PATH',params['LSF_CONF_PATH'])
        
    # Check user, create it if not existing
    glib.export('GLITE_USER',params['lb.user'])
    if glib.verbose:
        print "Checking local logger user"
    uid,gid = glib.add_user(os.environ['GLITE_USER'])
        
    glib.export('GLITE_HOST_CERT',"%s/%s/hostcert.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))
    glib.export('GLITE_HOST_KEY',"%s/%s/hostkey.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))

    if not params.has_key('X509_VOMS_DIR'):
        params['X509_VOMS_DIR'] = '/etc/grid-security/vomsdir'
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])

    if os.environ.has_key('GLOBUS_TCP_PORT_RANGE'):
        #os.environ['GLOBUS_TCP_PORT_RANGE'] = '"%s"' % os.environ['GLOBUS_TCP_PORT_RANGE']
        glib.export('GLOBUS_TCP_PORT_RANGE', os.environ['GLOBUS_TCP_PORT_RANGE'])
        low, high = os.environ['GLOBUS_TCP_PORT_RANGE'].split(' ')
        glib.export('MYPROXY_TCP_PORT_RANGE', "%s,%s" % (low, high))

    # bin and lib paths
    glib.addEnvPath("PATH","%s/bin:%s/bin:%s/bin:/opt/condor-c/libexec/glite:%s/externals/bin:%s/bin" \
        % (params['lcg.providers.location'],\
           os.environ['JAVA_HOME'],\
           os.environ['GLOBUS_LOCATION'],\
           os.environ['GLITE_LOCATION'],\
           os.environ['GLITE_LOCATION']))
    glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib:%s/externals/lib:%s/lib" \
        % (params['lcg.providers.location'],\
           os.environ['GLOBUS_LOCATION'],\
           os.environ['GLITE_LOCATION'],\
           os.environ['GLITE_LOCATION']))

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl" % os.environ['GPT_LOCATION'])

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of ConfigParams
    params = ConfigParams()

    # Get an instance of the library classes
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    loadDefaults(params)
    verboseCLI = 0
    try:
        opts, args = glib.getopt(sys.argv[1:], 'nchv', ['noiptables', 'checkconf', 'help', 'version', 'configure', 'ldif', 'start', 'stop', 'status', 'service=', 'siteconfig=','verbose','cemon'])
        for o, a in opts:

            if o == "--verbose":
                verboseCLI = 1
                glib.verbose = 1

            if o == "--siteconfig":
                params['site.config.url'] = a
                break
                
    except getopt.GetoptError:
        pass
        
    if glib.loadConfiguration(["%s/../glite-ce.cfg.xml" % glib.getScriptPath(), \
            "%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()],params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    verbose = 0
    if verboseCLI:
        verbose = 1
    else:
        if params.has_key('glite.installer.verbose'):
            if params['glite.installer.verbose'] == "true":
                verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # Instantiate the service classes
    service = glite_ce()
    service.verbose = verbose
    
    # Instantiate the condor class
    condor = Condor(params)
    condor.verbose = verbose
    
    # Instantiate the globus class
    globus = Globus(params)
    globus.verbose = verbose
    
    # Instantiate the tomcat class
    tomcat = Tomcat5(params)
    tomcat.verbose = verbose
    
    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose
    
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'nchv', ['noiptables', 'checkconf', 'help', 'version', 'configure', 'ldif', 'start', 'stop', 'status', 'service=', 'siteconfig=','verbose','cemon'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o == "--service":
            service.servicecommand = a

        if o in ("-n", "--noiptables"):
            service.iptables_config = 0

        if o == "--cemon":
            service.servicecommand = 'cemon'

    # Check cli options
    for o, a in opts:

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            print
            rgmaServicetool.showServices()
            sys.exit(0)

        if o == "--configure":

            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running CE services...")
            service.stop()
            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running CE services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "\nBefore using the %s services make sure that" % service.friendly_name
                print "the host certificate of each used VOMS server is installed in"
                print "%s\n" % os.environ['X509_VOMS_DIR']
                print "You can now start the service using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running CE services...")
                service.stop()
                
                #    Added 05/04/2006 Robert
                #    Service can be correctly configured although warnings were issued
                glib.registerService()
                
                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s service" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--ldif":
            service.copyright()
            service.showVersion()
            sys.exit(service.generateCEInfoProvidersFile())

        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--status":
            sys.exit(service.status())

            
            
